@@ -321,7 +321,95 @@ rb_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:7) */
+/* BOXED:BOXED (./rb-marshal.list:7) */
+extern void rb_marshal_BOXED__BOXED (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data);
+void
+rb_marshal_BOXED__BOXED (GClosure     *closure,
+                         GValue       *return_value,
+                         guint         n_param_values,
+                         const GValue *param_values,
+                         gpointer      invocation_hint,
+                         gpointer      marshal_data)
+{
+  typedef gpointer (*GMarshalFunc_BOXED__BOXED) (gpointer     data1,
+                                                 gpointer     arg_1,
+                                                 gpointer     data2);
+  register GMarshalFunc_BOXED__BOXED callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gpointer v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOXED__BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_boxed (param_values + 1),
+                       data2);
+
+  g_value_take_boxed (return_value, v_return);
+}
+
+/* VOID:BOXED,BOXED (./rb-marshal.list:8) */
+extern void rb_marshal_VOID__BOXED_BOXED (GClosure     *closure,
+                                          GValue       *return_value,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint,
+                                          gpointer      marshal_data);
+void
+rb_marshal_VOID__BOXED_BOXED (GClosure     *closure,
+                              GValue       *return_value,
+                              guint         n_param_values,
+                              const GValue *param_values,
+                              gpointer      invocation_hint,
+                              gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__BOXED_BOXED) (gpointer     data1,
+                                                  gpointer     arg_1,
+                                                  gpointer     arg_2,
+                                                  gpointer     data2);
+  register GMarshalFunc_VOID__BOXED_BOXED callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__BOXED_BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_boxed (param_values + 1),
+            g_marshal_value_peek_boxed (param_values + 2),
+            data2);
+}
+
+/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:9) */
 extern void rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -368,7 +456,7 @@ rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,POINTER (./rb-marshal.list:8) */
+/* VOID:BOXED,POINTER (./rb-marshal.list:10) */
 extern void rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -411,7 +499,52 @@ rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,ULONG (./rb-marshal.list:9) */
+/* VOID:BOXED,STRING,BOXED (./rb-marshal.list:11) */
+extern void rb_marshal_VOID__BOXED_STRING_BOXED (GClosure     *closure,
+                                                 GValue       *return_value,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint,
+                                                 gpointer      marshal_data);
+void
+rb_marshal_VOID__BOXED_STRING_BOXED (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__BOXED_STRING_BOXED) (gpointer     data1,
+                                                         gpointer     arg_1,
+                                                         gpointer     arg_2,
+                                                         gpointer     arg_3,
+                                                         gpointer     data2);
+  register GMarshalFunc_VOID__BOXED_STRING_BOXED callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__BOXED_STRING_BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_boxed (param_values + 1),
+            g_marshal_value_peek_string (param_values + 2),
+            g_marshal_value_peek_boxed (param_values + 3),
+            data2);
+}
+
+/* VOID:BOXED,ULONG (./rb-marshal.list:12) */
 extern void rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -454,7 +587,7 @@ rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:DOUBLE,LONG (./rb-marshal.list:10) */
+/* VOID:DOUBLE,LONG (./rb-marshal.list:13) */
 extern void rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -497,7 +630,7 @@ rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT,DOUBLE (./rb-marshal.list:11) */
+/* VOID:INT,INT,DOUBLE (./rb-marshal.list:14) */
 extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -542,7 +675,7 @@ rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,POINTER (./rb-marshal.list:12) */
+/* VOID:INT,POINTER (./rb-marshal.list:15) */
 extern void rb_marshal_VOID__INT_POINTER (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -585,7 +718,7 @@ rb_marshal_VOID__INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT (./rb-marshal.list:13) */
+/* VOID:OBJECT,INT,INT (./rb-marshal.list:16) */
 extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -630,7 +763,7 @@ rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:14) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:17) */
 extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -681,7 +814,7 @@ rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:15) */
+/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:18) */
 extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -726,7 +859,7 @@ rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT (./rb-marshal.list:16) */
+/* VOID:POINTER,INT (./rb-marshal.list:19) */
 extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -769,7 +902,7 @@ rb_marshal_VOID__POINTER_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT,POINTER (./rb-marshal.list:17) */
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:20) */
 extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -814,7 +947,7 @@ rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER (./rb-marshal.list:18) */
+/* VOID:POINTER,POINTER (./rb-marshal.list:21) */
 extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -857,7 +990,7 @@ rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,ULONG (./rb-marshal.list:19) */
+/* VOID:POINTER,ULONG (./rb-marshal.list:22) */
 extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -900,7 +1033,7 @@ rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,DOUBLE (./rb-marshal.list:20) */
+/* VOID:STRING,DOUBLE (./rb-marshal.list:23) */
 extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -943,7 +1076,7 @@ rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./rb-marshal.list:21) */
+/* VOID:STRING,INT (./rb-marshal.list:24) */
 extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -986,7 +1119,7 @@ rb_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,POINTER (./rb-marshal.list:22) */
+/* VOID:STRING,POINTER (./rb-marshal.list:25) */
 extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1029,7 +1162,7 @@ rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING (./rb-marshal.list:23) */
+/* VOID:STRING,STRING (./rb-marshal.list:26) */
 extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1072,7 +1205,7 @@ rb_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING (./rb-marshal.list:24) */
+/* VOID:STRING,STRING,STRING (./rb-marshal.list:27) */
 extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -1117,7 +1250,7 @@ rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:25) */
+/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:28) */
 extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -1166,7 +1299,7 @@ rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:ULONG,FLOAT (./rb-marshal.list:26) */
+/* VOID:ULONG,FLOAT (./rb-marshal.list:29) */
 extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -1209,7 +1342,7 @@ rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:27) */
+/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:30) */
 extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1252,7 +1385,7 @@ rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:28) */
+/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:31) */
 extern void rb_marshal_VOID__STRING_STRING_POINTER_POINTER (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
