@@ -53,11 +53,21 @@ rb_null_function (gpointer dummy)
 	return NULL;
 }
 
+gpointer
+rb_copy_function (gpointer data)
+{
+	return data;
+}
+
+
 int
 rb_gvalue_compare (GValue *a, GValue *b)
 {
 	int retval;
 	const char *stra, *strb;
+
+	if (G_VALUE_TYPE (a) != G_VALUE_TYPE (b))
+		return -1;
 	
 	switch (G_VALUE_TYPE (a))
 	{
@@ -175,8 +185,14 @@ rb_gvalue_compare (GValue *a, GValue *b)
 		retval = g_utf8_collate (stra, strb);
 		break;
 	case G_TYPE_POINTER:
+		retval = (g_value_get_pointer (a) != g_value_get_pointer (b));
+		break;
 	case G_TYPE_BOXED:
+		retval = (g_value_get_boxed (a) != g_value_get_boxed (b));
+		break;
 	case G_TYPE_OBJECT:
+		retval = (g_value_get_object (a) != g_value_get_object (b));
+		break;
 	default:
 		g_assert_not_reached ();
 		retval = 0;
@@ -381,6 +397,27 @@ rb_is_main_thread (void)
 	}
 }
 
+static gboolean
+purge_useless_threads (gpointer data)
+{
+	g_thread_pool_stop_unused_threads ();
+	return TRUE;
+}
+
+
+static GStaticRecMutex rb_gdk_mutex;
+
+static void
+_threads_enter (void)
+{
+	g_static_rec_mutex_lock (&rb_gdk_mutex);
+}
+
+static void
+_threads_leave (void)
+{
+	g_static_rec_mutex_unlock (&rb_gdk_mutex);
+}
 
 void
 rb_threads_init (void)
@@ -388,10 +425,12 @@ rb_threads_init (void)
 	private_is_primary_thread = g_private_new (NULL);
 	g_private_set (private_is_primary_thread, GUINT_TO_POINTER (1));
 
-	/* not really necessary, but in case it does something besides
-	 * set up lock functions some day..
-	 */
+	g_static_rec_mutex_init (&rb_gdk_mutex);
+	gdk_threads_set_lock_functions (_threads_enter, _threads_leave);
 	gdk_threads_init ();
+
+	/* purge useless thread-pool threads occasionally */
+	g_timeout_add (30 * 1000, purge_useless_threads, NULL);
 }
 
 gchar **
@@ -418,7 +457,7 @@ rb_string_split_words (const gchar *string)
 	while (*cur_read) {
 		switch (g_unichar_type (*cur_read)) {
 		case G_UNICODE_UNASSIGNED:
-			g_warning ("unassigned unicode character type found");
+			rb_debug ("unassigned unicode character type found");
 			/* fall through */
 		case G_UNICODE_CONTROL:
 		case G_UNICODE_FORMAT:
@@ -544,7 +583,7 @@ rb_search_fold (const char *original)
 			break;
 
 		case G_UNICODE_UNASSIGNED:
-			g_warning ("unassigned unicode character type found");
+			rb_debug ("unassigned unicode character type found");
 			/* fall through */
 
 		default:
@@ -818,6 +857,7 @@ rb_value_array_append_data (GValueArray *array, GType type, ...)
 	g_value_init (&val, type);
 	G_VALUE_COLLECT (&val, va, 0, &err);
 	g_value_array_append (array, &val);
+	g_value_unset (&val);
 
 	if (err)
 		rb_debug ("unable to collect GValue: %s", err);
