@@ -63,7 +63,7 @@ rhythmdb_query_copy (GPtrArray *array)
 
 	if (!array)
 		return NULL;
-	
+
 	ret = g_ptr_array_sized_new (array->len);
 	rhythmdb_query_concatenate (ret, array);
 
@@ -78,7 +78,7 @@ rhythmdb_query_concatenate (GPtrArray *query1, GPtrArray *query2)
 	g_assert (query2);
 	if (!query2)
 		return;
-	
+
 	for (i = 0; i < query2->len; i++) {
 		RhythmDBQueryData *data = g_ptr_array_index (query2, i);
 		RhythmDBQueryData *new_data = g_new0 (RhythmDBQueryData, 1);
@@ -95,14 +95,13 @@ rhythmdb_query_concatenate (GPtrArray *query1, GPtrArray *query2)
 	}
 }
 
-
 GPtrArray *
 rhythmdb_query_parse_valist (RhythmDB *db, va_list args)
 {
 	RhythmDBQueryType query;
 	GPtrArray *ret = g_ptr_array_new ();
 	char *error;
-	
+
 	while ((query = va_arg (args, RhythmDBQueryType)) != RHYTHMDB_QUERY_END) {
 		RhythmDBQueryData *data = g_new0 (RhythmDBQueryData, 1);
 		data->type = query;
@@ -110,7 +109,7 @@ rhythmdb_query_parse_valist (RhythmDB *db, va_list args)
 		case RHYTHMDB_QUERY_DISJUNCTION:
 			break;
 		case RHYTHMDB_QUERY_SUBQUERY:
-			data->subquery = va_arg (args, GPtrArray *);
+			data->subquery = rhythmdb_query_copy (va_arg (args, GPtrArray *));
 			break;
 		case RHYTHMDB_QUERY_PROP_EQUALS:
 		case RHYTHMDB_QUERY_PROP_LIKE:
@@ -123,7 +122,7 @@ rhythmdb_query_parse_valist (RhythmDB *db, va_list args)
 		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN:
 		case RHYTHMDB_QUERY_PROP_YEAR_EQUALS:
 		case RHYTHMDB_QUERY_PROP_YEAR_GREATER:
-		case RHYTHMDB_QUERY_PROP_YEAR_LESS:  
+		case RHYTHMDB_QUERY_PROP_YEAR_LESS:
 			data->propid = va_arg (args, guint);
 			data->val = g_new0 (GValue, 1);
 			g_value_init (data->val, rhythmdb_get_property_type (db, data->propid));
@@ -176,7 +175,7 @@ rhythmdb_query_parse_valist (RhythmDB *db, va_list args)
  * 	a) that match the query "subquery", or
  * 	b) that have "cat" in their title, or
  * 	c) have a rating of at least 2.5, and a play count of at most 10
- * 
+ *
  * Returns: a the newly created query. It must be freed with rhythmdb_query_free()
  **/
 GPtrArray *
@@ -194,7 +193,6 @@ rhythmdb_query_parse (RhythmDB *db, ...)
 	return ret;
 }
 
-
 /**
  * rhythmdb_query_append:
  * @db: a #RhythmDB instance
@@ -237,7 +235,7 @@ rhythmdb_query_free (GPtrArray *query)
 
 	if (query == NULL)
 		return;
-	
+
 	for (i = 0; i < query->len; i++) {
 		RhythmDBQueryData *data = g_ptr_array_index (query, i);
 		switch (data->type) {
@@ -257,7 +255,7 @@ rhythmdb_query_free (GPtrArray *query)
 		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN:
 		case RHYTHMDB_QUERY_PROP_YEAR_EQUALS:
 		case RHYTHMDB_QUERY_PROP_YEAR_GREATER:
-		case RHYTHMDB_QUERY_PROP_YEAR_LESS:  
+		case RHYTHMDB_QUERY_PROP_YEAR_LESS:
 			g_value_unset (data->val);
 			g_free (data->val);
 			break;
@@ -271,6 +269,47 @@ rhythmdb_query_free (GPtrArray *query)
 	g_ptr_array_free (query, TRUE);
 }
 
+static char *
+prop_gvalue_to_string (RhythmDB *db,
+		       RhythmDBPropType propid,
+		       GValue *val)
+{
+	/* special-case some properties */
+	switch (propid) {
+	case RHYTHMDB_PROP_TYPE:
+		{
+			RhythmDBEntryType type = g_value_get_pointer (val);
+			return g_strdup (type->name);
+		}
+		break;
+	default:
+		break;
+	}
+
+	/* otherwise just convert numbers to strings */
+	switch (G_VALUE_TYPE (val)) {
+	case G_TYPE_STRING:
+		return g_value_dup_string (val);
+	case G_TYPE_BOOLEAN:
+		return g_strdup_printf ("%d", g_value_get_boolean (val));
+	case G_TYPE_INT:
+		return g_strdup_printf ("%d", g_value_get_int (val));
+	case G_TYPE_LONG:
+		return g_strdup_printf ("%ld", g_value_get_long (val));
+	case G_TYPE_ULONG:
+		return g_strdup_printf ("%lu", g_value_get_ulong (val));
+	case G_TYPE_UINT64:
+		return g_strdup_printf ("%" G_GUINT64_FORMAT, g_value_get_uint64 (val));
+	case G_TYPE_FLOAT:
+		return g_strdup_printf ("%f", g_value_get_float (val));
+	case G_TYPE_DOUBLE:
+		return g_strdup_printf ("%f", g_value_get_double (val));
+	default:
+		g_assert_not_reached ();
+		return NULL;
+	}
+}
+
 static void
 write_encoded_gvalue (RhythmDB *db,
 		      xmlNodePtr node,
@@ -280,52 +319,7 @@ write_encoded_gvalue (RhythmDB *db,
 	char *strval = NULL;
 	xmlChar *quoted;
 
-	/* special-case some properties */
-	switch (propid) {
-	case RHYTHMDB_PROP_TYPE:
-		{
-			RhythmDBEntryType type = g_value_get_pointer (val);
-			strval = g_strdup (type->name);
-		}
-		break;
-	default:
-		break;
-	}
-
-	/* otherwise just convert numbers to strings */
-	if (!strval) {
-		switch (G_VALUE_TYPE (val)) {
-		case G_TYPE_STRING:
-			strval = g_value_dup_string (val);
-			break;
-		case G_TYPE_BOOLEAN:
-			strval = g_strdup_printf ("%d", g_value_get_boolean (val));
-			break;
-		case G_TYPE_INT:
-			strval = g_strdup_printf ("%d", g_value_get_int (val));
-			break;
-		case G_TYPE_LONG:
-			strval = g_strdup_printf ("%ld", g_value_get_long (val));
-			break;
-		case G_TYPE_ULONG:
-			strval = g_strdup_printf ("%lu", g_value_get_ulong (val));
-			break;
-		case G_TYPE_UINT64:
-			strval = g_strdup_printf ("%" G_GUINT64_FORMAT, g_value_get_uint64 (val));
-			break;
-		case G_TYPE_FLOAT:
-			strval = g_strdup_printf ("%f", g_value_get_float (val));
-			break;
-		case G_TYPE_DOUBLE:
-			strval = g_strdup_printf ("%f", g_value_get_double (val));
-			break;
-		default:
-			g_assert_not_reached ();
-			strval = NULL;
-			break;
-		}
-	}
-
+	strval = prop_gvalue_to_string (db, propid, val);
 	quoted = xmlEncodeEntitiesReentrant (NULL, BAD_CAST strval);
 	g_free (strval);
 
@@ -340,7 +334,7 @@ rhythmdb_read_encoded_property (RhythmDB *db,
 				GValue *val)
 {
 	g_value_init (val, rhythmdb_get_property_type (db, propid));
-	
+
 	switch (G_VALUE_TYPE (val)) {
 	case G_TYPE_STRING:
 		g_value_set_string (val, content);
@@ -355,12 +349,24 @@ rhythmdb_read_encoded_property (RhythmDB *db,
 		g_value_set_uint64 (val, g_ascii_strtoull (content, NULL, 10));
 		break;
 	case G_TYPE_DOUBLE:
-		g_value_set_double (val, g_ascii_strtod (content, NULL));
+		{
+			gdouble d;
+			char *end;
+
+			d = g_ascii_strtod (content, &end);
+			if (*end != '\0') {
+				/* conversion wasn't entirely successful.
+				 * try locale-aware strtod().
+				 */
+				d = strtod (content, NULL);
+			}
+			g_value_set_double (val, d);
+		}
 		break;
 	case G_TYPE_POINTER:
 		if (propid == RHYTHMDB_PROP_TYPE) {
 			RhythmDBEntryType entry_type;
-			entry_type = rhythmdb_entry_type_get_by_name (content);
+			entry_type = rhythmdb_entry_type_get_by_name (db, content);
 			if (entry_type != RHYTHMDB_ENTRY_TYPE_INVALID) {
 				g_value_set_pointer (val, entry_type);
 				break;
@@ -369,11 +375,11 @@ rhythmdb_read_encoded_property (RhythmDB *db,
 				/* Fall through */
 			}
 		}
-		/* Falling through on purpose to get an assert for unexpected 
-		 * cases 
+		/* Falling through on purpose to get an assert for unexpected
+		 * cases
 		 */
 	default:
-		g_warning ("Attempt to read '%s' of unhandled type %s", 
+		g_warning ("Attempt to read '%s' of unhandled type %s",
 			   rhythmdb_nice_elt_name_from_propid (db, propid),
 			   g_type_name (G_VALUE_TYPE (val)));
 		g_assert_not_reached ();
@@ -381,7 +387,6 @@ rhythmdb_read_encoded_property (RhythmDB *db,
 	}
 }
 
-
 void
 rhythmdb_query_serialize (RhythmDB *db, GPtrArray *query,
 			  xmlNodePtr parent)
@@ -392,7 +397,7 @@ rhythmdb_query_serialize (RhythmDB *db, GPtrArray *query,
 
 	for (i = 0; i < query->len; i++) {
 		RhythmDBQueryData *data = g_ptr_array_index (query, i);
-		
+
 		switch (data->type) {
 		case RHYTHMDB_QUERY_SUBQUERY:
 			subnode = xmlNewChild (node, NULL, RB_PARSE_SUBQUERY, NULL);
@@ -448,7 +453,7 @@ rhythmdb_query_serialize (RhythmDB *db, GPtrArray *query,
 			xmlSetProp (subnode, RB_PARSE_PROP, rhythmdb_nice_elt_name_from_propid (db, data->propid));
 			write_encoded_gvalue (db, subnode, data->propid, data->val);
 			break;
-		case RHYTHMDB_QUERY_PROP_YEAR_LESS:  
+		case RHYTHMDB_QUERY_PROP_YEAR_LESS:
 			subnode = xmlNewChild (node, NULL, RB_PARSE_YEAR_LESS, NULL);
 			xmlSetProp (subnode, RB_PARSE_PROP, rhythmdb_nice_elt_name_from_propid (db, data->propid));
 			write_encoded_gvalue (db, subnode, data->propid, data->val);
@@ -463,7 +468,7 @@ rhythmdb_query_serialize (RhythmDB *db, GPtrArray *query,
 			xmlSetProp (subnode, RB_PARSE_PROP, rhythmdb_nice_elt_name_from_propid (db, data->propid));
 			write_encoded_gvalue (db, subnode, data->propid, data->val);
 			break;
-		}		
+		}
 	}
 }
 
@@ -474,7 +479,7 @@ rhythmdb_query_deserialize (RhythmDB *db, xmlNodePtr parent)
 	xmlNodePtr child;
 
 	g_assert (!xmlStrcmp (parent->name, RB_PARSE_CONJ));
-	
+
 	for (child = parent->children; child; child = child->next) {
 		RhythmDBQueryData *data;
 
@@ -489,7 +494,7 @@ rhythmdb_query_deserialize (RhythmDB *db, xmlNodePtr parent)
 			subquery = child->children;
 			while (xmlNodeIsText (subquery))
 				subquery = subquery->next;
-			
+
 			data->subquery = rhythmdb_query_deserialize (db, subquery);
 		} else if (!xmlStrcmp (child->name, RB_PARSE_DISJ)) {
 			data->type = RHYTHMDB_QUERY_DISJUNCTION;
@@ -502,23 +507,32 @@ rhythmdb_query_deserialize (RhythmDB *db, xmlNodePtr parent)
 		} else if (!xmlStrcmp (child->name, RB_PARSE_SUFFIX)) {
 			data->type = RHYTHMDB_QUERY_PROP_SUFFIX;
 		} else if (!xmlStrcmp (child->name, RB_PARSE_EQUALS)) {
-			if (!xmlStrcmp(xmlGetProp(child, RB_PARSE_PROP), 
-				       (xmlChar *)"date")) 
+			xmlChar* prop;
+
+			prop = xmlGetProp(child, RB_PARSE_PROP);
+			if (!xmlStrcmp(prop, (xmlChar *)"date"))
 				data->type = RHYTHMDB_QUERY_PROP_YEAR_EQUALS;
-			else 
+			else
 				data->type = RHYTHMDB_QUERY_PROP_EQUALS;
+			xmlFree (prop);
 		} else if (!xmlStrcmp (child->name, RB_PARSE_GREATER)) {
-			if (!xmlStrcmp(xmlGetProp(child, RB_PARSE_PROP), 
-				       (xmlChar *)"date")) 
+			xmlChar* prop;
+
+			prop = xmlGetProp(child, RB_PARSE_PROP);
+			if (!xmlStrcmp(prop, (xmlChar *)"date"))
 				data->type = RHYTHMDB_QUERY_PROP_YEAR_GREATER;
-			else 
+			else
 				data->type = RHYTHMDB_QUERY_PROP_GREATER;
+			xmlFree (prop);
 		} else if (!xmlStrcmp (child->name, RB_PARSE_LESS)) {
-			if (!xmlStrcmp(xmlGetProp(child, RB_PARSE_PROP), 
-				       (xmlChar *)"date")) 
+			xmlChar* prop;
+
+			prop = xmlGetProp(child, RB_PARSE_PROP);
+			if (!xmlStrcmp(prop, (xmlChar *)"date"))
 				data->type = RHYTHMDB_QUERY_PROP_YEAR_LESS;
-			else 
+			else
 				data->type = RHYTHMDB_QUERY_PROP_LESS;
+			xmlFree (prop);
 		} else if (!xmlStrcmp (child->name, RB_PARSE_CURRENT_TIME_WITHIN)) {
 			data->type = RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN;
 		} else if (!xmlStrcmp (child->name, RB_PARSE_CURRENT_TIME_NOT_WITHIN)) {
@@ -551,7 +565,7 @@ rhythmdb_query_deserialize (RhythmDB *db, xmlNodePtr parent)
 			content = (char *)xmlNodeGetContent (child);
 			rhythmdb_read_encoded_property (db, content, data->propid, data->val);
 			g_free (content);
-		} 
+		}
 
 		g_ptr_array_add (query, data);
 	}
@@ -572,7 +586,7 @@ rhythmdb_query_deserialize (RhythmDB *db, xmlNodePtr parent)
 void
 rhythmdb_query_preprocess (RhythmDB *db, GPtrArray *query)
 {
-	int i;	
+	int i;
 
 	if (query == NULL)
 		return;
@@ -580,7 +594,7 @@ rhythmdb_query_preprocess (RhythmDB *db, GPtrArray *query)
 	for (i = 0; i < query->len; i++) {
 		RhythmDBQueryData *data = g_ptr_array_index (query, i);
 		gboolean restart_criteria = FALSE;
-		
+
 		if (data->subquery) {
 			rhythmdb_query_preprocess (db, data->subquery);
 		} else switch (data->propid) {
@@ -620,19 +634,26 @@ rhythmdb_query_preprocess (RhythmDB *db, GPtrArray *query)
 				gulong year;
 
 				search_date = g_value_get_ulong (data->val);
-				g_date_set_julian (&date, search_date);
-				year = g_date_get_year (&date);
-				g_date_clear (&date, 1);
-
-				/* get Julian dates for beginning and end of year */
-				g_date_set_dmy (&date, 1, G_DATE_JANUARY, year);
-				begin = g_date_get_julian (&date);
-				g_date_clear (&date, 1);
-
-				/* and the day before the beginning of the next year */
-				g_date_set_dmy (&date, 1, G_DATE_JANUARY, year + 1);
-				end =  g_date_get_julian (&date) - 1;
 				
+				/* GDate functions don't handle Year="0", so we need to special case this */
+				if (search_date != 0) {
+					g_date_set_julian (&date, search_date);
+					year = g_date_get_year (&date);
+					g_date_clear (&date, 1);
+
+					/* get Julian dates for beginning and end of year */
+					g_date_set_dmy (&date, 1, G_DATE_JANUARY, year);
+					begin = g_date_get_julian (&date);
+					g_date_clear (&date, 1);
+
+					/* and the day before the beginning of the next year */
+					g_date_set_dmy (&date, 1, G_DATE_JANUARY, year + 1);
+					end =  g_date_get_julian (&date) - 1;
+				} else {
+					begin = 0;
+					end = 0;
+				}
+
 				switch (data->type)
 				{
 				case RHYTHMDB_QUERY_PROP_YEAR_EQUALS:
@@ -659,10 +680,10 @@ rhythmdb_query_preprocess (RhythmDB *db, GPtrArray *query)
 				default:
 					break;
 				}
-				
+
 				break;
 			}
-			
+
 			default:
 				break;
 		}
@@ -743,6 +764,92 @@ rhythmdb_query_is_time_relative (RhythmDB *db, GPtrArray *query)
 	return FALSE;
 }
 
+/**
+ * rhythmdb_query_to_string:
+ * @db: a #RhythmDB instance
+ * @query: a query.
+ *
+ * Returns a supposedly human-readable form of the query.
+ * This is only intended for debug usage.
+ **/
+char *
+rhythmdb_query_to_string (RhythmDB *db, GPtrArray *query)
+{
+	GString *buf;
+	int i;
+
+	buf = g_string_sized_new (100);
+	for (i = 0; i < query->len; i++) {
+		char *fmt = NULL;
+		RhythmDBQueryData *data = g_ptr_array_index (query, i);
+
+		switch (data->type) {
+		case RHYTHMDB_QUERY_SUBQUERY:
+			{
+				char *s;
+
+				s = rhythmdb_query_to_string (db, data->subquery);
+				g_string_append_printf (buf, "{ %s }",  s);
+				g_free (s);
+			}
+			break;
+		case RHYTHMDB_QUERY_PROP_LIKE:
+			fmt = "(%s =~ %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_NOT_LIKE:
+			fmt = "(%s !~ %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_PREFIX:
+			fmt = "(%s |< %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_SUFFIX:
+			fmt = "(%s >| %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_EQUALS:
+			fmt = "(%s == %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_YEAR_EQUALS:
+			fmt = "(year(%s) == %s)";
+			break;
+		case RHYTHMDB_QUERY_DISJUNCTION:
+			g_string_append_printf (buf, " || ");
+			break;
+		case RHYTHMDB_QUERY_END:
+			break;
+		case RHYTHMDB_QUERY_PROP_GREATER:
+			fmt = "(%s > %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_YEAR_GREATER:
+			fmt = "(year(%s) > %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_LESS:
+			fmt = "(%s < %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_YEAR_LESS:
+			fmt = "(year(%s) < %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN:
+			fmt = "(%s <> %s)";
+			break;
+		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN:
+			fmt = "(%s >< %s)";
+			break;
+		}
+
+		if (fmt) {
+			char *value;
+
+			value = prop_gvalue_to_string (db, data->propid, data->val);
+			g_string_append_printf (buf, fmt,
+						rhythmdb_nice_elt_name_from_propid (db, data->propid),
+						value);
+			g_free (value);
+			fmt = NULL;
+		}
+	}
+
+	return g_string_free (buf, FALSE);
+}
 
 GType
 rhythmdb_query_get_type (void)
@@ -757,4 +864,3 @@ rhythmdb_query_get_type (void)
 
 	return type;
 }
-
