@@ -21,7 +21,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -40,6 +40,7 @@ static GObject* rb_statusbar_construct (GType type,
 					GObjectConstructParam *construct_properties);
 static void rb_statusbar_class_init (RBStatusbarClass *klass);
 static void rb_statusbar_init (RBStatusbar *statusbar);
+static void rb_statusbar_dispose (GObject *object);
 static void rb_statusbar_finalize (GObject *object);
 static void rb_statusbar_set_property (GObject *object,
 				       guint prop_id,
@@ -61,7 +62,7 @@ struct RBStatusbarPrivate
 
         RhythmDB *db;
 
-        GtkActionGroup *actiongroup;
+        GtkUIManager *ui_manager;
         GtkTooltips *tooltips;
 
         GtkWidget *progress;
@@ -78,7 +79,7 @@ enum
 {
         PROP_0,
         PROP_DB,
-        PROP_ACTION_GROUP,
+        PROP_UI_MANAGER,
         PROP_SOURCE
 };
 
@@ -90,6 +91,7 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
         object_class->constructor = rb_statusbar_construct;
+        object_class->dispose = rb_statusbar_dispose;
         object_class->finalize = rb_statusbar_finalize;
 
         object_class->set_property = rb_statusbar_set_property;
@@ -110,11 +112,11 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
                                                               RB_TYPE_SOURCE,
                                                               G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
-                                         PROP_ACTION_GROUP,
-                                         g_param_spec_object ("action-group",
-                                                              "GtkActionGroup",
-                                                              "GtkActionGroup object",
-                                                              GTK_TYPE_ACTION_GROUP,
+                                         PROP_UI_MANAGER,
+                                         g_param_spec_object ("ui-manager",
+                                                              "GtkUIManager",
+                                                              "GtkUIManager object",
+                                                              GTK_TYPE_UI_MANAGER,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (klass, sizeof (RBStatusbarPrivate));
@@ -134,18 +136,17 @@ rb_statusbar_construct (GType                  type,
 					(type,
 					 n_construct_properties,
 					 construct_properties);
-        
+
         statusbar = RB_STATUSBAR (object);
 
         return object;
 }
 
-
 static void
 rb_statusbar_init (RBStatusbar *statusbar)
 {
-	statusbar->priv = G_TYPE_INSTANCE_GET_PRIVATE (statusbar, 
-						       RB_TYPE_STATUSBAR, 
+	statusbar->priv = G_TYPE_INSTANCE_GET_PRIVATE (statusbar,
+						       RB_TYPE_STATUSBAR,
 						       RBStatusbarPrivate);
 
         statusbar->priv->tooltips = gtk_tooltips_new ();
@@ -165,6 +166,41 @@ rb_statusbar_init (RBStatusbar *statusbar)
                             GTK_WIDGET (statusbar->priv->progress), FALSE, TRUE, 0);
 }
 
+static void
+rb_statusbar_dispose (GObject *object)
+{
+        RBStatusbar *statusbar;
+
+        g_return_if_fail (object != NULL);
+        g_return_if_fail (RB_IS_STATUSBAR (object));
+
+        statusbar = RB_STATUSBAR (object);
+
+        g_return_if_fail (statusbar->priv != NULL);
+
+	if (statusbar->priv->status_poll_id) {
+                g_source_remove (statusbar->priv->status_poll_id);
+		statusbar->priv->status_poll_id = 0;
+	}
+
+	if (statusbar->priv->db != NULL) {
+		g_object_unref (statusbar->priv->db);
+		statusbar->priv->db = NULL;
+	}
+
+	if (statusbar->priv->ui_manager != NULL) {
+		g_object_unref (statusbar->priv->ui_manager);
+		statusbar->priv->ui_manager = NULL;
+	}
+
+	if (statusbar->priv->selected_source != NULL) {
+		g_object_unref (statusbar->priv->selected_source);
+		statusbar->priv->selected_source = NULL;
+	}
+
+        G_OBJECT_CLASS (rb_statusbar_parent_class)->dispose (object);
+}
+
 static void
 rb_statusbar_finalize (GObject *object)
 {
@@ -180,13 +216,9 @@ rb_statusbar_finalize (GObject *object)
 	g_free (statusbar->priv->loading_text);
 	g_free (statusbar->priv->progress_text);
 
-        if (statusbar->priv->status_poll_id)
-                g_source_remove (statusbar->priv->status_poll_id);
-        
         G_OBJECT_CLASS (rb_statusbar_parent_class)->finalize (object);
 }
 
-
 typedef struct {
         GtkWidget *statusbar;
         char      *tooltip;
@@ -225,10 +257,10 @@ unset_statusbar_tooltip (GtkWidget *widget,
 }
 
 static void
-rb_statusbar_connect_action_group (RBStatusbar    *statusbar,
-                                   GtkAction      *action,
-                                   GtkWidget      *proxy,
-                                   GtkActionGroup *action_group)
+rb_statusbar_connect_ui_manager (RBStatusbar    *statusbar,
+				 GtkAction      *action,
+				 GtkWidget      *proxy,
+				 GtkUIManager   *ui_manager)
 {
         char *tooltip;
 
@@ -265,6 +297,7 @@ rb_statusbar_set_property (GObject *object,
         {
         case PROP_DB:
                 statusbar->priv->db = g_value_get_object (value);
+		g_object_ref (statusbar->priv->db);
                 statusbar->priv->status_poll_id
                         = g_idle_add ((GSourceFunc) poll_status, statusbar);
                 break;
@@ -273,10 +306,12 @@ rb_statusbar_set_property (GObject *object,
 			g_signal_handlers_disconnect_by_func (G_OBJECT (statusbar->priv->selected_source),
 							      G_CALLBACK (rb_statusbar_source_status_changed_cb),
 							      statusbar);
+			g_object_unref (statusbar->priv->selected_source);
                 }
-                
+
                 statusbar->priv->selected_source = g_value_get_object (value);
-                rb_debug ("selected source %p", g_value_get_object (value));
+                rb_debug ("selected source %p", statusbar->priv->selected_source);
+		g_object_ref (statusbar->priv->selected_source);
 
                 if (statusbar->priv->selected_source != NULL) {
 			g_signal_connect_object (G_OBJECT (statusbar->priv->selected_source),
@@ -287,17 +322,19 @@ rb_statusbar_set_property (GObject *object,
 		rb_statusbar_sync_status (statusbar);
 
                 break;
-        case PROP_ACTION_GROUP:
-                if (statusbar->priv->actiongroup) {
-                        g_signal_handlers_disconnect_by_func (G_OBJECT (statusbar->priv->actiongroup),
-                                                              G_CALLBACK (rb_statusbar_connect_action_group),
+        case PROP_UI_MANAGER:
+                if (statusbar->priv->ui_manager) {
+                        g_signal_handlers_disconnect_by_func (G_OBJECT (statusbar->priv->ui_manager),
+                                                              G_CALLBACK (rb_statusbar_connect_ui_manager),
                                                               statusbar);
+			g_object_unref (statusbar->priv->ui_manager);
                 }
-                statusbar->priv->actiongroup = g_value_get_object (value);
+                statusbar->priv->ui_manager = g_value_get_object (value);
+		g_object_ref (statusbar->priv->ui_manager);
 
-                g_signal_connect_object (statusbar->priv->actiongroup,
+                g_signal_connect_object (statusbar->priv->ui_manager,
                                          "connect-proxy",
-                                         G_CALLBACK (rb_statusbar_connect_action_group),
+                                         G_CALLBACK (rb_statusbar_connect_ui_manager),
                                          statusbar,
                                          G_CONNECT_SWAPPED);
                 break;
@@ -307,7 +344,7 @@ rb_statusbar_set_property (GObject *object,
         }
 }
 
-static void 
+static void
 rb_statusbar_get_property (GObject *object,
                               guint prop_id,
                               GValue *value,
@@ -323,8 +360,8 @@ rb_statusbar_get_property (GObject *object,
         case PROP_SOURCE:
                 g_value_set_object (value, statusbar->priv->selected_source);
                 break;
-        case PROP_ACTION_GROUP:
-                g_value_set_object (value, statusbar->priv->actiongroup);
+        case PROP_UI_MANAGER:
+                g_value_set_object (value, statusbar->priv->ui_manager);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -425,11 +462,12 @@ rb_statusbar_sync_status (RBStatusbar *status)
 }
 
 RBStatusbar *
-rb_statusbar_new (RhythmDB *db, GtkActionGroup *actions)
+rb_statusbar_new (RhythmDB *db,
+                  GtkUIManager *ui_manager)
 {
         RBStatusbar *statusbar = g_object_new (RB_TYPE_STATUSBAR,
                                                "db", db,
-                                               "action-group", actions,
+                                               "ui-manager", ui_manager,
                                                NULL);
 
         g_return_val_if_fail (statusbar->priv != NULL, NULL);
@@ -444,7 +482,7 @@ rb_statusbar_set_progress (RBStatusbar *statusbar, double progress, const char *
                 g_free (statusbar->priv->progress_text);
                 statusbar->priv->progress_text = NULL;
         }
-        
+
         if (progress > (0.0 - EPSILON)) {
                 statusbar->priv->progress_fraction = progress;
                 statusbar->priv->progress_changed = TRUE;
@@ -463,6 +501,6 @@ rb_statusbar_source_status_changed_cb (RBSource *source, RBStatusbar *statusbar)
 {
 	rb_debug ("source status changed");
         if (statusbar->priv->status_poll_id == 0)
-                statusbar->priv->status_poll_id = 
+                statusbar->priv->status_poll_id =
                         g_idle_add ((GSourceFunc) poll_status, statusbar);
 }
