@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -23,7 +23,7 @@
 #include <gtk/gtkuimanager.h>
 #include "eggtrayicon.h"
 #include "rhythmdb.h"
-#include "rb-remote-proxy.h"
+#include "rb-shell.h"
 
 #ifndef __RB_TRAY_ICON_H
 #define __RB_TRAY_ICON_H
@@ -53,8 +53,7 @@ typedef struct
 
 GType			rb_tray_icon_get_type	(void);
 
-RBTrayIcon *		rb_tray_icon_new	(GtkUIManager *mgr,
-						 RBRemoteProxy *remote);
+RBTrayIcon *		rb_tray_icon_new	(GtkUIManager *mgr, RBShell *shell);
 
 void                    rb_tray_icon_get_geom   (RBTrayIcon *icon, int *x, int *y, int *width, int *height);
 
@@ -62,7 +61,8 @@ void                    rb_tray_icon_notify     (RBTrayIcon *icon,
 						 guint timeout,
 						 const char *primary,
 						 GtkWidget *msgicon,
-						 const char *secondary);
+						 const char *secondary,
+						 gboolean requested);
 
 void                    rb_tray_icon_cancel_notify (RBTrayIcon *icon);
 
