@@ -157,7 +157,10 @@ static GtkRadioActionEntry rb_browser_source_radio_actions [] =
 	{ "BrowserSourceSearchTitles", NULL, N_("Titles"), NULL, N_("Search titles"), 3 }
 };
 
-static const GtkTargetEntry songs_view_drag_types[] = {{  "text/uri-list", 0, 0 }};
+static const GtkTargetEntry songs_view_drag_types[] = {
+	{ "application/x-rhythmbox-entry", 0, 0 },
+	{ "text/uri-list", 0, 1 }
+};
 
 enum
 {
@@ -380,12 +383,16 @@ rb_browser_source_constructor (GType type,
 								       G_N_ELEMENTS (rb_browser_source_actions),
 								       shell);
 
-	gtk_action_group_add_radio_actions (source->priv->action_group,
-					    rb_browser_source_radio_actions,
-					    G_N_ELEMENTS (rb_browser_source_radio_actions),
-					    0,
-					    (GCallback)search_action_changed,
-					    shell);
+	/* only add the actions if we haven't already */
+	if (gtk_action_group_get_action (source->priv->action_group,
+					 rb_browser_source_radio_actions[0].name) == NULL) {
+		gtk_action_group_add_radio_actions (source->priv->action_group,
+						    rb_browser_source_radio_actions,
+						    G_N_ELEMENTS (rb_browser_source_radio_actions),
+						    0,
+						    (GCallback)search_action_changed,
+						    shell);
+	}
 	g_object_unref (shell);
 
 	source->priv->paned = gtk_vpaned_new ();
@@ -448,7 +455,7 @@ rb_browser_source_constructor (GType type,
 	if (rb_browser_source_has_drop_support (source)) {
 		gtk_drag_dest_set (GTK_WIDGET (source->priv->songs),
 				   GTK_DEST_DEFAULT_ALL,
-				   songs_view_drag_types, 1,
+				   songs_view_drag_types, G_N_ELEMENTS (songs_view_drag_types),
 				   GDK_ACTION_COPY | GDK_ACTION_MOVE);	/* really accept move actions? */
 
 		/* set up drag and drop for the song tree view.
