@@ -56,7 +56,6 @@ static void rb_removable_media_source_get_property (GObject *object,
 
 static void impl_delete_thyself (RBSource *source);
 
-
 typedef struct
 {
 	GnomeVFSVolume *volume;
@@ -71,8 +70,6 @@ enum
 	PROP_VOLUME,
 };
 
-
-
 static void
 rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 {
@@ -120,7 +117,7 @@ static GObject *
 rb_removable_media_source_constructor (GType type, guint n_construct_properties,
 				       GObjectConstructParam *construct_properties)
 {
-	GObject *source; 
+	GObject *source;
 	GnomeVFSVolume *volume;
 	GnomeVFSDrive *drive;
 	char *display_name;
@@ -159,8 +156,7 @@ rb_removable_media_source_constructor (GType type, guint n_construct_properties,
 	return source;
 }
 
-
-static void 
+static void
 rb_removable_media_source_dispose (GObject *object)
 {
 	RBRemovableMediaSourcePrivate *priv = REMOVABLE_MEDIA_SOURCE_GET_PRIVATE (object);
@@ -221,12 +217,14 @@ impl_delete_thyself (RBSource *source)
 	RBShell *shell;
 	RhythmDBEntryType entry_type;
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-	g_object_unref (G_OBJECT (shell));
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "db", &db, NULL);
+	g_object_unref (shell);
 
-	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
+	g_object_get (source, "entry-type", &entry_type, NULL);
 	rhythmdb_entry_delete_by_type (db, entry_type);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+
 	rhythmdb_commit (db);
 	g_object_unref (db);
 }
